@@ -6,7 +6,6 @@
 #include <cmConfigure.h>
 
 #include "cmCTestHandlerCommand.h"
-#include "cmTypeMacro.h"
 
 #include <string>
 
@@ -39,8 +38,6 @@ public:
    */
   std::string GetName() const CM_OVERRIDE { return "ctest_configure"; }
 
-  cmTypeMacro(cmCTestConfigureCommand, cmCTestHandlerCommand);
-
 protected:
   cmCTestGenericHandler* InitializeHandler() CM_OVERRIDE;
 
