@@ -6,7 +6,6 @@
 #include <cmConfigure.h>
 
 #include "cmCTestTestCommand.h"
-#include "cmTypeMacro.h"
 
 #include <string>
 
@@ -21,7 +20,7 @@ class cmCommand;
 class cmCTestMemCheckCommand : public cmCTestTestCommand
 {
 public:
-  cmCTestMemCheckCommand() {}
+  cmCTestMemCheckCommand();
 
   /**
    * This is a virtual constructor for the command.
@@ -39,10 +38,16 @@ public:
    */
   std::string GetName() const CM_OVERRIDE { return "ctest_memcheck"; }
 
-  cmTypeMacro(cmCTestMemCheckCommand, cmCTestTestCommand);
-
 protected:
   cmCTestGenericHandler* InitializeActualHandler() CM_OVERRIDE;
+
+  void ProcessAdditionalValues(cmCTestGenericHandler* handler) CM_OVERRIDE;
+
+  enum
+  {
+    ctm_DEFECT_COUNT = ctt_LAST,
+    ctm_LAST
+  };
 };
 
 #endif
