@@ -7,7 +7,6 @@
 
 #include "cmCTest.h"
 #include "cmCTestHandlerCommand.h"
-#include "cmTypeMacro.h"
 
 #include <set>
 #include <string>
@@ -55,7 +54,7 @@ public:
    */
   std::string GetName() const CM_OVERRIDE { return "ctest_submit"; }
 
-  cmTypeMacro(cmCTestSubmitCommand, cmCTestHandlerCommand);
+  typedef cmCTestHandlerCommand Superclass;
 
 protected:
   cmCTestGenericHandler* InitializeHandler() CM_OVERRIDE;
