@@ -1,19 +1,15 @@
-/*============================================================================
-  CMake - Cross Platform Makefile Generator
-  Copyright 2000-2009 Kitware, Inc., Insight Software Consortium
+/* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
+   file Copyright.txt or https://cmake.org/licensing for details.  */
+#ifndef cmCursesStringWidget_h
+#define cmCursesStringWidget_h
 
-  Distributed under the OSI-approved BSD License (the "License");
-  see accompanying file Copyright.txt for details.
-
-  This software is distributed WITHOUT ANY WARRANTY; without even the
-  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-  See the License for more information.
-============================================================================*/
-#ifndef __cmCursesStringWidget_h
-#define __cmCursesStringWidget_h
+#include <cmConfigure.h>
 
+#include "cmCursesStandardIncludes.h"
 #include "cmCursesWidget.h"
 
+#include <string>
+
 class cmCursesMainForm;
 
 /** \class cmCursesStringWidget
@@ -32,14 +28,14 @@ public:
    * when this widget has focus. Returns true if the input was
    * handled.
    */
-  virtual bool HandleInput(int& key, cmCursesMainForm* fm, WINDOW* w);
+  bool HandleInput(int& key, cmCursesMainForm* fm, WINDOW* w) CM_OVERRIDE;
 
   /**
    * Set/Get the string.
    */
-  void SetString(const char* value);
+  void SetString(const std::string& value);
   const char* GetString();
-  virtual const char* GetValue();
+  const char* GetValue() CM_OVERRIDE;
 
   /**
    * Set/Get InEdit flag. Can be used to tell the widget to leave
@@ -61,7 +57,7 @@ public:
    * in the toolbar and return true. Otherwise, return false
    * and the parent widget will print.
    */
-  virtual bool PrintKeys();
+  bool PrintKeys() CM_OVERRIDE;
 
 protected:
   cmCursesStringWidget(const cmCursesStringWidget& from);
@@ -73,4 +69,4 @@ protected:
   bool Done;
 };
 
-#endif // __cmCursesStringWidget_h
+#endif // cmCursesStringWidget_h
