@@ -3,8 +3,14 @@
 #ifndef cmAuxSourceDirectoryCommand_h
 #define cmAuxSourceDirectoryCommand_h
 
+#include <cmConfigure.h>
+#include <string>
+#include <vector>
+
 #include "cmCommand.h"
 
+class cmExecutionStatus;
+
 /** \class cmAuxSourceDirectoryCommand
  * \brief Specify auxiliary source code directories.
  *
@@ -33,8 +39,6 @@ public:
    * The name of the command as specified in CMakeList.txt.
    */
   std::string GetName() const CM_OVERRIDE { return "aux_source_directory"; }
-
-  cmTypeMacro(cmAuxSourceDirectoryCommand, cmCommand);
 };
 
 #endif
