@@ -3,22 +3,13 @@
 #ifndef cmDefinitions_h
 #define cmDefinitions_h
 
-#include <cmConfigure.h>
-
-#include "cmLinkedTree.h"
+#include <cmConfigure.h> // IWYU pragma: keep
 
 #include <string>
 #include <vector>
 
-#if defined(CMAKE_BUILD_WITH_CMAKE)
-#ifdef CMake_HAVE_CXX_UNORDERED_MAP
-#include <unordered_map>
-#else
-#include "cmsys/hash_map.hxx"
-#endif
-#else
-#include <map>
-#endif
+#include "cmLinkedTree.h"
+#include "cm_unordered_map.hxx"
 
 /** \class cmDefinitions
  * \brief Store a scope of variable definitions for CMake language.
@@ -85,15 +76,7 @@ private:
   };
   static Def NoDef;
 
-#if defined(CMAKE_BUILD_WITH_CMAKE)
-#ifdef CMake_HAVE_CXX_UNORDERED_MAP
-  typedef std::unordered_map<std::string, Def> MapType;
-#else
-  typedef cmsys::hash_map<std::string, Def> MapType;
-#endif
-#else
-  typedef std::map<std::string, Def> MapType;
-#endif
+  typedef CM_UNORDERED_MAP<std::string, Def> MapType;
   MapType Map;
 
   static Def const& GetInternal(const std::string& key, StackIter begin,
