@@ -1,20 +1,15 @@
-/*============================================================================
-  CMake - Cross Platform Makefile Generator
-  Copyright 2000-2009 Kitware, Inc., Insight Software Consortium
-
-  Distributed under the OSI-approved BSD License (the "License");
-  see accompanying file Copyright.txt for details.
-
-  This software is distributed WITHOUT ANY WARRANTY; without even the
-  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-  See the License for more information.
-============================================================================*/
+/* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
+   file Copyright.txt or https://cmake.org/licensing for details.  */
 #ifndef cmFileCommand_h
 #define cmFileCommand_h
 
+#include <cmConfigure.h>
+#include <string>
+#include <vector>
+
 #include "cmCommand.h"
 
-struct cmFileInstaller;
+class cmExecutionStatus;
 
 /** \class cmFileCommand
  * \brief Command for manipulation of files
@@ -26,29 +21,24 @@ public:
   /**
    * This is a virtual constructor for the command.
    */
-  virtual cmCommand* Clone()
-    {
-    return new cmFileCommand;
-    }
+  cmCommand* Clone() CM_OVERRIDE { return new cmFileCommand; }
 
   /**
    * This is called when the command is first encountered in
    * the CMakeLists.txt file.
    */
-  virtual bool InitialPass(std::vector<std::string> const& args,
-                           cmExecutionStatus &status);
+  bool InitialPass(std::vector<std::string> const& args,
+                   cmExecutionStatus& status) CM_OVERRIDE;
 
   /**
    * This determines if the command is invoked when in script mode.
    */
-  virtual bool IsScriptable() const { return true; }
+  bool IsScriptable() const CM_OVERRIDE { return true; }
 
   /**
    * The name of the command as specified in CMakeList.txt.
    */
-  virtual std::string GetName() const { return "file";}
-
-  cmTypeMacro(cmFileCommand, cmCommand);
+  std::string GetName() const CM_OVERRIDE { return "file"; }
 
 protected:
   bool HandleRename(std::vector<std::string> const& args);
@@ -78,11 +68,9 @@ protected:
   bool HandleLockCommand(std::vector<std::string> const& args);
 
 private:
-  void AddEvaluationFile(const std::string &inputName,
-                         const std::string &outputExpr,
-                         const std::string &condition,
-                         bool inputIsContent);
+  void AddEvaluationFile(const std::string& inputName,
+                         const std::string& outputExpr,
+                         const std::string& condition, bool inputIsContent);
 };
 
-
 #endif
