@@ -3,8 +3,15 @@
 #ifndef cmFindProgramCommand_h
 #define cmFindProgramCommand_h
 
+#include <cmConfigure.h>
+#include <string>
+#include <vector>
+
 #include "cmFindBase.h"
 
+class cmCommand;
+class cmExecutionStatus;
+
 /** \class cmFindProgramCommand
  * \brief Define a command to search for an executable program.
  *
@@ -39,8 +46,6 @@ public:
    */
   std::string GetName() const CM_OVERRIDE { return "find_program"; }
 
-  cmTypeMacro(cmFindProgramCommand, cmFindBase);
-
 private:
   std::string FindProgram();
   std::string FindNormalProgram();
