@@ -3,8 +3,14 @@
 #ifndef cmGetPropertyCommand_h
 #define cmGetPropertyCommand_h
 
+#include <cmConfigure.h>
+#include <string>
+#include <vector>
+
 #include "cmCommand.h"
 
+class cmExecutionStatus;
+
 class cmGetPropertyCommand : public cmCommand
 {
 public:
@@ -29,8 +35,6 @@ public:
    */
   std::string GetName() const CM_OVERRIDE { return "get_property"; }
 
-  cmTypeMacro(cmGetPropertyCommand, cmCommand);
-
 private:
   enum OutType
   {
