@@ -3,8 +3,14 @@
 #ifndef cmMathCommand_h
 #define cmMathCommand_h
 
+#include <cmConfigure.h>
+#include <string>
+#include <vector>
+
 #include "cmCommand.h"
 
+class cmExecutionStatus;
+
 /// Mathematical expressions: math(EXPR ...) command.
 class cmMathCommand : public cmCommand
 {
@@ -31,8 +37,6 @@ public:
    */
   std::string GetName() const CM_OVERRIDE { return "math"; }
 
-  cmTypeMacro(cmMathCommand, cmCommand);
-
 protected:
   bool HandleExprCommand(std::vector<std::string> const& args);
 };
