@@ -3,8 +3,14 @@
 #ifndef cmSiteNameCommand_h
 #define cmSiteNameCommand_h
 
+#include <cmConfigure.h>
+#include <string>
+#include <vector>
+
 #include "cmCommand.h"
 
+class cmExecutionStatus;
+
 /** \class cmSiteNameCommand
  * \brief site_name command
  *
@@ -34,8 +40,6 @@ public:
    * The name of the command as specified in CMakeList.txt.
    */
   std::string GetName() const CM_OVERRIDE { return "site_name"; }
-
-  cmTypeMacro(cmSiteNameCommand, cmCommand);
 };
 
 #endif
