@@ -3,8 +3,16 @@
 #ifndef cmTargetCompileDefinitionsCommand_h
 #define cmTargetCompileDefinitionsCommand_h
 
+#include <cmConfigure.h>
+#include <string>
+#include <vector>
+
 #include "cmTargetPropCommandBase.h"
 
+class cmCommand;
+class cmExecutionStatus;
+class cmTarget;
+
 class cmTargetCompileDefinitionsCommand : public cmTargetPropCommandBase
 {
 public:
@@ -31,8 +39,6 @@ public:
     return "target_compile_definitions";
   }
 
-  cmTypeMacro(cmTargetCompileDefinitionsCommand, cmTargetPropCommandBase);
-
 private:
   void HandleImportedTarget(const std::string& tgt) CM_OVERRIDE;
   void HandleMissingTarget(const std::string& name) CM_OVERRIDE;
