@@ -3,8 +3,16 @@
 #ifndef cmTargetCompileFeaturesCommand_h
 #define cmTargetCompileFeaturesCommand_h
 
+#include <cmConfigure.h>
+#include <string>
+#include <vector>
+
 #include "cmTargetPropCommandBase.h"
 
+class cmCommand;
+class cmExecutionStatus;
+class cmTarget;
+
 class cmTargetCompileFeaturesCommand : public cmTargetPropCommandBase
 {
   cmCommand* Clone() CM_OVERRIDE { return new cmTargetCompileFeaturesCommand; }
@@ -14,8 +22,6 @@ class cmTargetCompileFeaturesCommand : public cmTargetPropCommandBase
 
   std::string GetName() const CM_OVERRIDE { return "target_compile_features"; }
 
-  cmTypeMacro(cmTargetCompileFeaturesCommand, cmTargetPropCommandBase);
-
 private:
   void HandleImportedTarget(const std::string& tgt) CM_OVERRIDE;
   void HandleMissingTarget(const std::string& name) CM_OVERRIDE;
