@@ -23,13 +23,16 @@
 #error Expected STRING_PROP3
 #endif
 
-template<bool test>
+template <bool test>
 struct CMakeStaticAssert;
 
-template<>
-struct CMakeStaticAssert<true> {};
+template <>
+struct CMakeStaticAssert<true>
+{
+};
 
-enum {
+enum
+{
   NumericMaxTest1 = sizeof(CMakeStaticAssert<NUMBER_MAX_PROP1 == 100>),
   NumericMaxTest2 = sizeof(CMakeStaticAssert<NUMBER_MAX_PROP2 == 250>),
   NumericMinTest1 = sizeof(CMakeStaticAssert<NUMBER_MIN_PROP1 == 50>),
@@ -40,8 +43,14 @@ enum {
 
 #include "iface2.h"
 
-int main(int argc, char **argv)
+int foo();
+#ifdef _WIN32
+__declspec(dllimport)
+#endif
+  int bar();
+
+int main(int argc, char** argv)
 {
   Iface2 if2;
-  return if2.foo();
+  return if2.foo() + foo() + bar();
 }
