@@ -3,13 +3,12 @@
 
 #include <stdio.h>
 
-int main ()
+int main()
 {
-  if ( LibCxx1Class::Method() != 2.0 )
-    {
+  if (LibCxx1Class::Method() != 2.0) {
     printf("Problem with libcxx1\n");
     return 1;
-    }
+  }
 #ifdef TEST_FLAG_3
   return 0;
 #else
